      subroutine cmpt_ort(fft_klh, hkl_max, fft_ann, t_analy, 
     $     t_fftanaly, mem_cmpt, ctf_val, abs_ctf, disef, fno,
     $     stb_ornt_stb_orgn, stb_ornt_new_orgn,
     $     new_ornt_stb_orgn, new_ornt_new_orgn,
     $     flp_ornt_stb_orgn, flp_ornt_new_orgn)

C     This is the key subroutine that performs the main function of
C     program POR - refinement of the particle origins and
C     orientations. The root process reads in the frames from the image
C     file and distributes them to the other processes. After all
C     processes have finished calculating the updated particle
C     information (orientation, origin, score), the root process
C     collects the results and writes out an updated parameter file with
C     the same base name as the original file and an updated suffix.
C
C     Arguments
C
C     fft_klh     - 3D FFT of model intent (in)
C     hkl_max     - array bound for interpolant array (in)
C     fft_ann     - array bound for fft_hkl (in)
C     t_analy     - cummulative timer (inout)
C     t_fftanaly  - cummulative timer for FFTs (inout)
C     mem_cmpt    - total memory allocated by routine (out)
C     ctf_val     - filtered, adjusted, weighted inverse of CTF (in)
C     abs_ctf     - filtered absolute value of CTF (in)
C     disef       - weighting function (in)
C     fno         - image file number (in)

      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      
C     Subroutine arguments
      integer, intent(in) :: hkl_max, fft_ann, fno
      integer, intent(inout) :: stb_ornt_stb_orgn, stb_ornt_new_orgn
      integer, intent(inout) :: new_ornt_stb_orgn, new_ornt_new_orgn
      integer, intent(inout) :: flp_ornt_stb_orgn, flp_ornt_new_orgn
      complex, intent(in) :: fft_klh(-fft_ann:fft_ann, 
     +     -fft_ann:fft_ann, 0:fft_ann)
      real, intent(in) :: ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in) :: abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in) :: disef(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      double precision, intent(inout) :: t_analy, t_fftanaly
      double precision, intent(out) :: mem_cmpt
      
C     Local variables

      integer process_total
      integer process_stb_ornt_stb_orgn
      integer process_stb_ornt_new_orgn
      integer process_new_ornt_stb_orgn
      integer process_new_ornt_new_orgn
      integer process_flp_ornt_stb_orgn
      integer process_flp_ornt_new_orgn

      integer mgraph_total
      integer mgraph_stb_ornt_stb_orgn
      integer mgraph_stb_ornt_new_orgn
      integer mgraph_new_ornt_stb_orgn
      integer mgraph_new_ornt_new_orgn
      integer mgraph_flp_ornt_stb_orgn
      integer mgraph_flp_ornt_new_orgn

      logical stable_orient, stable_origin, stable_hand

      character(len=1) :: orient_flag, origin_flag
      character(len=1) :: flag_new(2, max_views/num_nodes+1),
     +     flag_out(2, max_views)

      real theta_save, phi_save, omega_save

      integer view, i, j, cc, max_node_eq
      integer :: ishift, maxshift = 10
      integer mem_size, start_view, end_view, num_views
      integer theta_step, phi_step, omega_step
      integer ort_out, viewno, centeru, centerv
      integer best_step(3), min_cen(2), sendcount(0:num_nodes),
     +     disp(0:num_nodes), minu, minv
      integer alloc_stat

      integer, parameter :: ptle_select_flag = 0
      
      real :: zero = 0.0
      real :: r4max = huge(r4max)
      real center(2), imgmagfactor,
     +     theta_test, phi_test, omega_test,
     +     num_annul,value,min_value, min_annul, du, dv
      real rot_mat(3,3), 
     +     dist(-Nangle:Nangle,-Nangle:Nangle,-Nangle:Nangle),
     +     distcen(-Ncenter:Ncenter,-Ncenter:Ncenter)
      real orient(0:9, max_views), 
     +     orient_new(0:7, max_views/num_nodes+1),
     +     orient_out(0:7, max_views)
      real theta, phi, omega, delta_theta, delta_phi, delta_omega
      
      real, allocatable :: rffta_omega(:)
      real, allocatable :: pixels_array(:,:,:)
      real, allocatable :: unit_array(:,:)
      complex, allocatable :: fc_2d(:,:)
      complex, allocatable :: fc_2d_intp(:,:)
      complex, allocatable :: fc_2d_bak(:,:)
      
      double precision elapsed_time
      integer*8 clock1, clock2
      
      double precision :: mem_orient = 0.0D0, mem_rffta_omega = 0.0D0,
     +     mem_fc_2d = 0.0D0 
      
      character(len=12) :: subname = 'cmpt_ort'

      process_stb_ornt_stb_orgn = 0
      process_stb_ornt_new_orgn = 0
      process_new_ornt_stb_orgn = 0
      process_new_ornt_new_orgn = 0
      process_flp_ornt_stb_orgn = 0
      process_flp_ornt_new_orgn = 0

      mgraph_stb_ornt_stb_orgn = 0
      mgraph_stb_ornt_new_orgn = 0
      mgraph_new_ornt_stb_orgn = 0
      mgraph_new_ornt_new_orgn = 0
      mgraph_flp_ornt_stb_orgn = 0
      mgraph_flp_ornt_new_orgn = 0

C     Allocate memory for dynamic arrays
      mem_size        = ( 1 + nfc_2d_row ) * ( 1 + 2 * nfc_2d_col )
      mem_rffta_omega = dble(4 * nrffta_omega)
      mem_orient      = dble(4*10*max_views)
      mem_fc_2d       = dble( 24 * mem_size )
      
      mem_cmpt = 3*mem_orient + mem_rffta_omega + mem_fc_2d
     +     + (2*Nangle+1)*(2*Nangle+1)*(2*Nangle+1)
     +     + (2*Ncenter+1)*(2*Ncenter+1)*(2*Ncenter+1)
      
      allocate(rffta_omega(nrffta_omega),
     +     fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     fc_2d_intp(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col), 
     +     fc_2d_bak(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     unit_array(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     pixels_array(npix_row, npix_col, view_perread),
     +     stat = alloc_stat)
      
C     Initialize unit array using Fortrn 90 array syntax
      unit_array = 1.0

      if(alloc_stat > 0) then
         call error_stop(progname, subname, ' Memory allocation failed')
      endif

C     Initialize the array rffta_omega that is used by FFT library
      call vrffti(nrffta_col, rffta_omega)

      num_views = 0 
      viewno = 0 

      call system_clock(clock1)
      
C     Read particle parameters and distribute to all processes
      if(iam == 0) then
         call readorient(orient, max_views, ort_file_name,
     +        num_views, npix_row, npix_col, ptle_select_flag)
         write(*,*)
         write(*,*) 'Orientation file ', trim(ort_file_name)
      endif
      
      call mpi_bcast(num_views, 1, mpi_integer,
     +     0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_BCAST num_views')
      
      call mpi_bcast(orient, 10*max_views, mpi_real,
     +     0, mpi_comm_world, mpi_err)
      if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_BCAST orient')
      
      call system_clock(clock2)
      t_analy = t_analy + elapsed_time(clock1, clock2)
      
      num_annul = real(nrffta_row)*angstrom_per_pix/res_max 
      min_annul = real(nrffta_row)*angstrom_per_pix/res_min 
      if (num_annul >= nrffta_row/2.) 
     +     call error_stop(progname,subname, 
     +     'Resolution is too high, smaller than 2*angstrom_per_pix')
      
C     we seperate all the views by view_perread, 
C     each time we read at most view_perread * num_nodes 
C     pixels and deal with them
      
      dO j = 0, (num_views-1)/(view_perread*num_nodes)
         
         call system_clock(clock1)
         
         start_view = 1 + j * view_perread*num_nodes
         end_view   = start_view + view_perread*num_nodes - 1
         if (end_view > num_views) end_view = num_views
         
C     the pix_file_name has been opened in aamain
C     Read the current end_view-start_view+1 pixel frames.
         
         call set_indices(num_nodes, start_view, end_view,
     +        frst_lst_view, max_node_eq )
C     CALL READ_FILES
C     ***************
C     processor 0 read pixels_array, then send to processor i
         do i = num_nodes - 1 , 0 , -1
            if (frst_lst_view(2, i) >= frst_lst_view(1, i)) then
               call read_files(pixels_array, frst_lst_view(1,i),
     +              frst_lst_view(2,i), i, orient)
            endif
         enddo
         
         call system_clock(clock2)
         t_analy = t_analy + elapsed_time(clock1, clock2)
         
C     do with each view and pixel in  per read
         
         start_view = frst_lst_view(1,iam)
         end_view   = frst_lst_view(2,iam)
         
         do view = start_view, end_view
C     
C     Get the orientation angles and center of next pixel frame
C     Also changes OMEGA from the range (0,360) deg to (-90,270) deg.
C     Because the orient used by BIO not consistent with defination 

            theta        = orient(1,view)
            phi          = orient(2,view)
            omega        = orient(3,view)
            center(1)    = orient(4,view)
            center(2)    = orient(5,view)
            imgmagfactor = orient(6,view)

            theta_save = theta
            phi_save   = phi
            omega_save = omega

            process_total = process_total + 1

C     adjust the imgmagfactor according to its own pixel_size
            imgmagfactor = imgmagfactor * PIXSIZ_OUT / angstrom_per_pix
C     
C------------------------------------------------------------
C     Compute Discrete Fourier Transform of pixel values:
C     CALL FFTANALY
C     *************
            call system_clock(clock1)
            
            call fft_2dfft( pixels_array(1, 1, view-start_view+1),
     +           npix_row, npix_col, center,
     +           nrffta_omega, rffta_omega, nrffta_row, 
     +           nrffta_col, nfc_2d_row, 
     +           nfc_2d_col, fc_2d )

C     Need to apply tempfac to images
            call maptempfac(fc_2d, 0, nfc_2d_row, -nfc_2d_col,
     $           nfc_2d_col, 1, 1, tempfac, pixsiz_out, units)

            du = center(1) - real(int(center(1)))
            dv = center(2) - real(int(center(2)))
            call shift_fft(fc_2d, nfc_2d_row, nfc_2d_col, du, dv)
            
C     backup fc_2d
            fc_2d_bak = fc_2d  ! f90 array syntax
            fc_2d_intp = 0.    ! f90 array syntax

            call system_clock(clock2)
            t_fftanaly = t_fftanaly + elapsed_time(clock1, clock2)

            min_value = R4MAX


C     ----------- Refine particle orientation -----------
C
C     Search a region of orientation space, centered on the current
C     orientation, for a new model projection that has the best
C     agreement (lowest score) with the particle image. The number of
C     steps taken in each direction for each angle is nangle. If the new
C     optimal orientation is more than nangle/2 steps from the starting
C     point in any direction, perform another search centered on this
C     new orientation. To minimize the number of comparisons that need
C     to be calculated, the array DIST containing the scores is
C     recentered so that previously calculated scores can be reused. The
C     outer loop over ishift sets a limit on the number of times that
C     search can be restarted.
C
C     Note that in this convention, the angle omega has to be set to
C     (360-omega-90), or equivalently, -(omega+90).

C     Do a very minimal local search first to determine whether or not
C     to perform a more extensive search
            
            dist = R4MAX
            best_step(1) = 0
            best_step(2) = 0
            best_step(3) = 0

            if(quick_search == 1) then
               do omega_step = -1, 1
                  do phi_step = -1, 1
                     do theta_step = -1, 1
                        
                        delta_theta = dangle * theta_step
                        delta_phi   = dangle * phi_step
                        delta_omega = dangle * omega_step
                        
                        theta_test = delta_theta + theta
                        phi_test   = delta_phi   + phi
                        omega_test = delta_omega + omega
                        
                        call crowther_to_matrix(theta_test, 
     $                       -phi_test ,-omega_test, rot_mat)
                        call interpl_3d(fft_klh, hkl_max, 
     +                       min_annul, num_annul,
     +                       rot_mat, fc_2d_intp, imgmagfactor, 
     +                       fft_ann) 
                        
                        fc_2d_intp = fc_2d_intp * ctf_val
                        
                        call imgcomport(fc_2d_intp, hkl_max, 
     +                       fc_2d, disef, abs_ctf, min_annul, 
     +                       num_annul, value, funcmode)
                        dist(theta_step, phi_step, omega_step)=value
                        
                        if (value < min_value) then
                           min_value = value
                           best_step(1) = theta_step
                           best_step(2) = phi_step
                           best_step(3) = omega_step
                        endif
                     enddo
                  enddo
               enddo
            endif
               
            if(quick_search   .ne. 1 .or. best_step(1) .ne. 0 .or.
     $           best_step(2) .ne. 0 .or. best_step(3) .ne. 0) then
               
               min_value = R4MAX
               dist = R4MAX
               
               do ishift = 1, maxshift

                  best_step(1) = 0
                  best_step(2) = 0
                  best_step(3) = 0

                  do omega_step = -Nangle, Nangle  
                     do phi_step = -Nangle, Nangle  
                        do theta_step = -Nangle, Nangle  
                           
                           delta_theta = dangle * theta_step
                           delta_phi   = dangle * phi_step
                           delta_omega = dangle * omega_step
                           
                           theta_test = delta_theta + theta
                           phi_test   = delta_phi   + phi
                           omega_test = delta_omega + omega
                           
                           if (dist(theta_step,phi_step,omega_step) >= 
     +                          R4MAX-1) then
                              call crowther_to_matrix(theta_test, 
     $                             -phi_test ,-omega_test, rot_mat)
                              call interpl_3d(fft_klh, hkl_max, 
     +                             min_annul, num_annul,
     +                             rot_mat, fc_2d_intp, imgmagfactor, 
     +                             fft_ann) 
                              
                              fc_2d_intp = fc_2d_intp * ctf_val
                              
                              call imgcomport(fc_2d_intp, hkl_max, 
     +                             fc_2d, disef, abs_ctf, min_annul, 
     +                             num_annul, value, funcmode)
                              dist(theta_step,phi_step,omega_step)=value
                              
                              if (value < min_value) then
                                 min_value = value
                                 best_step(1) = theta_step
                                 best_step(2) = phi_step
                                 best_step(3) = omega_step
                              endif
                           endif
                        enddo
                     enddo
                  enddo
                  
                  theta = theta + best_step(1) * dangle
                  phi   = phi   + best_step(2) * dangle
                  omega = omega + best_step(3) * dangle
                  
C     If the new optimal orientation is sufficiently far from the
C     starting orientation, recenter the array DIST and start a new
C     search. Otherwise, finished with orientation refinement.
                  
                  if ( abs(real(best_step(1))) > Nangle*0.5 .or. 
     +                 abs(real(best_step(2))) > Nangle*0.5 .or.
     +                 abs(real(best_step(3))) > Nangle*0.5 ) then
                     call recenter_array_3d(dist,Nangle,best_step,R4MAX)
                  else
                     exit
                  endif
               enddo
            endif

            if(theta_save == theta .and. phi_save == phi .and.
     $           omega_save == omega) then
               stable_orient = .true.
            else
               stable_orient = .false.
            endif

C     ----------- Check handedness of orientation -----------
C
C     Compare current orientation with the corresponding orientation
C     with the opposite handedness (theta, -phi, omega+180). Switch
C     handedness of the opposite handed orientation has a better match
C     with the model.

            theta_save = theta
            phi_save   = phi
            omega_save = omega

            call crowther_to_matrix(theta, phi, -omega+180, rot_mat)
            call interpl_3d(fft_klh, hkl_max,
     +           min_annul, num_annul,
     +           rot_mat, fc_2d_intp, imgmagfactor, fft_ann) 

            fc_2d_intp = fc_2d_intp * ctf_val

            call imgcomport(fc_2d_intp, hkl_max, 
     +           fc_2d, disef, abs_ctf, min_annul, num_annul,
     +           value, funcmode)
            if (value < min_value) then
               min_value = value
               phi = - phi
               omega = omega + 180.0
            endif

            if(theta_save == theta .and. phi_save == phi .and. 
     $           omega_save == omega) then
               stable_hand = .true.
            else
               stable_hand = .false.
            endif
               
C     ----------- Return orientation to asymmetric unit -----------
C
            if ( symm_code > 1) then
               call to_asym_unit(symm_code, theta, phi, omega)
            endif

C     ----------- Refine particle origin -----------
C
C     Search a region of origin space, centered on the current origin,
C     for a new model projection that has the best agreement (lowest
C     score) with the particle image. The number of steps taken in each
C     direction for each coordinate is ncenter. If the new optimal
C     origin is more than ncenter/2 steps from the starting point in any
C     direction, perform another search centered on this new origin. To
C     minimize the number of comparisons that need to be calculated, the
C     array DISTCEN containing the scores is recentered so that
C     previously calculated scores can be reused. The outer loop over
C     ishift sets a limit on the number of times that search can be
C     restarted.

            call crowther_to_matrix(theta, -phi, -omega, rot_mat)
            call interpl_3d(fft_klh, hkl_max,
     +           min_annul, num_annul,
     +           rot_mat, fc_2d_intp, imgmagfactor, fft_ann) 

            fc_2d_intp = fc_2d_intp * ctf_val

            
            distcen = R4MAX    ! f90 array syntax
            
            min_value = R4MAX
            minu      = 0
            minv      = 0

            do ishift = 1, maxshift
               
               min_cen(1) = 0
               min_cen(2) = 0
               
               do centerv = -Ncenter, Ncenter 
                  dv = Dcenter * (centerv + minv)
                  do centeru = -Ncenter, Ncenter 
                     if (distcen(centeru,centerv) >= R4MAX-1) then
                        du = Dcenter * (centeru + minu)
                        fc_2d = fc_2d_bak
                        call shift_fft(fc_2d,
     +                       nfc_2d_row, nfc_2d_col, du, dv)

                        call imgcomport(fc_2d_intp, hkl_max, 
     +                       fc_2d, unit_array, abs_ctf, min_annul,
     +                       num_annul, value, 1)

                        distcen(centeru,centerv) = value
                        if (value < min_value) then
                           min_value = value
                           min_cen(1) = centeru
                           min_cen(2) = centerv
                        endif
                     endif
                  enddo
               enddo
            
C     If the new optimal origin is sufficiently far from the starting
C     origin, recenter the array DISTCEN and start a new
C     search. Otherwise, finished with origin refinement.

               if ( abs(real(min_cen(1))) > Ncenter*0.5 .or. 
     +              abs(real(min_cen(2))) > Ncenter*0.5 ) then
                  minu = minu + min_cen(1) 
                  minv = minv + min_cen(2)
                  call recenter_array_2d(distcen,Ncenter,min_cen,R4MAX)
               else
                  exit
               endif

            enddo

            du = (minu+min_cen(1))*Dcenter
            dv = (minv+min_cen(2))*Dcenter

            if(du == 0 .and. dv == 0) then
               stable_origin = .true.
            else
               stable_origin = .false.
            endif

            orient_flag = 'S'
            origin_flag = 'S'

            if(  stable_orient .and. 
     $           stable_hand   .and. 
     $           stable_origin) then
               process_stb_ornt_stb_orgn = process_stb_ornt_stb_orgn + 1
            endif

            if(  stable_orient .and. 
     $           stable_hand   .and. 
     $           .not. stable_origin) then
               process_stb_ornt_new_orgn = process_stb_ornt_new_orgn + 1
               origin_flag = 'N'
            endif

            if(  .not. stable_orient .and. 
     $           stable_origin) then
               process_new_ornt_stb_orgn = process_new_ornt_stb_orgn + 1
               orient_flag = 'N'
            endif

            if(  .not. stable_orient .and. 
     $           .not. stable_origin) then
               process_new_ornt_new_orgn = process_new_ornt_new_orgn + 1
               orient_flag = 'N'
               origin_flag = 'N'
            endif

            if(  stable_orient .and. 
     $           .not. stable_hand   .and. 
     $           stable_origin) then
               process_flp_ornt_stb_orgn = process_flp_ornt_stb_orgn + 1
               orient_flag = 'F'
            endif

            if(  stable_orient .and. 
     $           .not. stable_hand   .and. 
     $           .not. stable_origin) then
               process_flp_ornt_new_orgn = process_flp_ornt_new_orgn + 1
               orient_flag = 'F'
               origin_flag = 'N'
            endif

            fc_2d = fc_2d_bak
            call shift_fft(fc_2d, nfc_2d_row, nfc_2d_col, du, dv)

C     compute the CC value for the DFT and best CUT
            call imgcomport(fc_2d_intp, hkl_max, 
     +           fc_2d, disef, abs_ctf, min_annul, num_annul,
     +           min_value, funcmode)
            
C     update new center and CC value
            viewno               = viewno +1
            orient_new(0,viewno) = orient(0, view)
            orient_new(1,viewno) = theta
            orient_new(2,viewno) = phi
            orient_new(3,viewno) = omega
            orient_new(4,viewno) = orient(4,view)+ du
            orient_new(5,viewno) = orient(5,view)+ dv
            orient_new(6,viewno) = orient(6,view)
            orient_new(7,viewno) = min_value
            flag_new(1,viewno)   = orient_flag
            flag_new(2,viewno)   = origin_flag

         enddo
      enddo             
      
C     Gather results onto root process
      call set_indices(num_nodes, 1, num_views, frst_lst_view, 
     +     max_node_eq )

      call mpi_reduce(process_stb_ornt_stb_orgn, 
     $     mgraph_stb_ornt_stb_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)
      
      call mpi_reduce(process_stb_ornt_new_orgn, 
     $     mgraph_stb_ornt_new_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)
      
      call mpi_reduce(process_new_ornt_stb_orgn, 
     $     mgraph_new_ornt_stb_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)
      
      call mpi_reduce(process_new_ornt_new_orgn, 
     $     mgraph_new_ornt_new_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)
      
      call mpi_reduce(process_flp_ornt_stb_orgn, 
     $     mgraph_flp_ornt_stb_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)
      
      call mpi_reduce(process_flp_ornt_new_orgn, 
     $     mgraph_flp_ornt_new_orgn,
     $     1, mpi_integer, mpi_sum, 0, mpi_comm_world, mpi_err)

      mgraph_total = mgraph_stb_ornt_stb_orgn
     $     + mgraph_stb_ornt_new_orgn
     $     + mgraph_new_ornt_stb_orgn
     $     + mgraph_new_ornt_new_orgn
     $     + mgraph_flp_ornt_stb_orgn
     $     + mgraph_flp_ornt_new_orgn

      stb_ornt_stb_orgn = stb_ornt_stb_orgn + mgraph_stb_ornt_stb_orgn
      stb_ornt_new_orgn = stb_ornt_new_orgn + mgraph_stb_ornt_new_orgn
      new_ornt_stb_orgn = new_ornt_stb_orgn + mgraph_new_ornt_stb_orgn
      new_ornt_new_orgn = new_ornt_new_orgn + mgraph_new_ornt_new_orgn
      flp_ornt_stb_orgn = flp_ornt_stb_orgn + mgraph_flp_ornt_stb_orgn
      flp_ornt_new_orgn = flp_ornt_new_orgn + mgraph_flp_ornt_new_orgn

      do i = 0, num_nodes-1
         sendcount(i)=(frst_lst_view(2,i)-frst_lst_view(1,i)+1)*8
         if (i == 0) then 
            disp(0) = 0
         else
            disp(i) = disp(i-1) + sendcount(i-1)
         endif
      enddo
      
      call mpi_gatherv(orient_new, sendcount(iam), mpi_real,
     $     orient_out, sendcount, disp, mpi_real,
     $     0, mpi_comm_world, mpi_err)

      if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_gatherv orient_out')

      do i = 0, num_nodes-1
         sendcount(i)=(frst_lst_view(2,i)-frst_lst_view(1,i)+1)*2
         if (i == 0) then 
            disp(0) = 0
         else
            disp(i) = disp(i-1) + sendcount(i-1)
         endif
      enddo
      
      call mpi_gatherv(flag_new, sendcount(iam), mpi_character,
     $     flag_out, sendcount, disp, mpi_character,
     $     0, mpi_comm_world, mpi_err)

      if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_gatherv orient_out')

C     Write out new particle parameter file from root process
      if (iam == 0) then

         write(*,*) 'micrograph total ptles                       ',
     $        mgraph_total
         write(*,*) 'micrograph stable orientation and origin     ',
     $        mgraph_stb_ornt_stb_orgn
         write(*,*) 'micrograph stable orientation and new origin ',
     $        mgraph_stb_ornt_new_orgn
         write(*,*) 'micrograph new orientation and stable origin ',
     $        mgraph_new_ornt_stb_orgn
         write(*,*) 'micrograph new orientation and new origin    ',
     $        mgraph_new_ornt_new_orgn
         write(*,*) 'micrograph hand flip and stable origin       ',
     $        mgraph_flp_ornt_stb_orgn
         write(*,*) 'micrograph hand flip and new origin          ',
     $        mgraph_flp_ornt_new_orgn

         call increment_fname(ort_file_name, ort_out_name)
         ort_out = 11
         OPEN (UNIT=ort_out, FILE=ort_out_name)
         write(ort_out,'(A)') trim(adjustl(pix_file_name))
         write(ort_out,1010) PIXSIZS(fno), units, VOLTS(fno), 
     +        AMPFACS(fno), DELF_MAJS(fno), DELF_MINS(fno), 
     +        ANG_MAJS(fno), CS_COEFFS(fno)
 1010    format(1X,F8.4,',',I2,',',F10.1,',',4(F10.4,','),F8.2) 

         do view = 1, num_views
            write(ort_out,1020) nint(orient_out(0, view)),
     +           (orient_out(cc, view),cc=1,7), zero, zero,
     +           flag_out(1, view), flag_out(2, view)
         enddo
         close(UNIT=ort_out)
 1020    format(I7,',',4(F8.3,','),F7.3,',',F6.3,',',
     +        F10.3,',',F5.1,',',F5.1,',',1x,A1,',',1x,A1)
      endif
      
C     Free memory allocated in this subroutine.
      deallocate(rffta_omega, fc_2d, fc_2d_intp, 
     +     fc_2d_bak, pixels_array, unit_array)
      
      return
      end subroutine cmpt_ort
