C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE GLOBAL     *****                  *
C  *                                                                          *
C  *    GLOBAL SEARCH FOR THETA,PHI,OMEGA,X,Y PARAMETERS (MODE.NE.3)          *
C  *                                                                          *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
C  *    MODIFIED  3-FEB-1998 BY T. BAKER (Parameters in arrays).              *
C  *    MODIFIED 27-FEB-1998 BY T. BAKER (Add UNIX compatibility).            *
C  *    MODIFIED  1-JUN-1998 BY TSB/SBW (Add CTF routines).                   *
C  *    LAST MODIFIED 24-JUL-02 BY TSB/RWA (Fix COMPARE_MODE=1 bugs).         *
C  ****************************************************************************
C
      SUBROUTINE GLOBAL (NCOLBIN, NROWBIN, NSECBIN, 
     1 NROT, NANNULI, NIDS_TOT, FIDS, IDS, TS, PS, OS,
     2 XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC)

      IMPLICIT NONE

      include 'mpif.h'
      include '../include/allprogs.inc'
      INCLUDE '../include/pifhead.inc'

      TYPE (PIFHEAD_T) MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD

      COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, MAG_NUM,
     $     MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90,
     $     PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO,
     $     ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2,
     $     RADIUS, DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES,
     $     IMG_FNAMES, QUICK_OMEGA

      integer*4 NCOLBIN, NROWBIN, NSECBIN, JCUT, QUICK_OMEGA
      integer*4 NROT, NANNULI, BIN, NTHE, NPHI, NVIEW_MOD
      integer*4 MODE, VERBOSE, SYM, IDIM1, IDIM2, MAG_NORM, MAG_NUM
      integer*4 ANN_LO, ANN_HI

      integer sendcount, basecount, remainder
      integer recvcounts(128), displs(128)
      integer prj_cnt, prj_size, nprjs, inode

      double precision elapsed_time
      double precision tg
      integer*8 clock1, clock2

      real*4    IMG_BKG, omega
      integer*4 NIDS_TOT, FIDS(NIDS_TOT), IDS(NIDS_TOT), COMPARE_MODE
      CHARACTER(len=256) PARAM_FNAMES(999), IMG_FNAMES(999)

      real*4, allocatable :: MOD_TS(:), MOD_PS(:)
      real*4, allocatable :: PRJ_AVG(:,:), PRJS(:,:,:)
      real*4, allocatable :: MAP3D(:,:,:), PRJ(:,:)

      real*4    tempVal
      integer*4 alloc_stat, iii, id, indx

      real*4  TS(NIDS_TOT), PS(NIDS_TOT), OS(NIDS_TOT)
      real*4  XS(NIDS_TOT), YS(NIDS_TOT), MAGS(NIDS_TOT)
      real*4  PFT_CCS(NIDS_TOT), PRJ_CCS(NIDS_TOT), CMP_CCS(NIDS_TOT)
      real*4  RES_MIN, RES_MAX
      real*4  DTHE, DPHI_90, THE1, THE2, PHI1, XCEN_MODEL, YCEN_MODEL
      real*4  PFTRAD_LO, PFTRAD_HI, DEL_ANG, FILT_FAC(10)
      real*4  MAG_CEN, MAG_STEP, SIGCUT, RADIUS
      real*4  PIXRES_MIN, PIXRES_MAX
      LOGICAL VAROPT
C     ----------------------------------------------------------------

      if(iam == 0) then
         tempVal =  (NCOLBIN*NROWBIN*NVIEW_MOD)
         tempVal =  (4*tempVal)
         if(tempVal .gt. 2.0**32) then
            print *, '*************************************************'
            print *, 'Warning - each process is asking for > 4 GB'
            print *, 'You''re asking for too much memory.'
            print *, '   Number of cols   - ', ncolbin
            print *, '   Number of rows   - ', nrowbin
            print *, '   Number of views  - ', nview_mod
            print *, '   == ', tempVal, ' total bytes requested.'
            print *, '          (4 bytes per pixel) '
         endif
      endif

      allocate(PRJ_AVG(NCOLBIN,NROWBIN),PRJS(NCOLBIN,NROWBIN,NVIEW_MOD),
     2     MOD_TS(NVIEW_MOD), MOD_PS(NVIEW_MOD), PRJ(NCOLBIN,NROWBIN),
     3     stat = alloc_stat)
      IF(alloc_stat .gt. 0) THEN
        PRINT *, '*************************************************'
        PRINT *, '* GETMEM1 memory allocation error: EXIT program *'
        CALL TSBEND
      ENDIF

C     Get THE1, THE2, & PHI1; 
C     Calculate grid of model THETAs and PHIs in search window
      CALL GET_T1T2P1 (SYM, THE1, THE2, PHI1)

      CALL CALC_MOD_TPS (MOD_TS, MOD_PS, THE1, THE2, PHI1, DTHE, 
     1  DPHI_90, NTHE, NPHI, NVIEW_MOD, SYM)

C     Get projections
C     MODE > 0, Calculate projections
C     MODE == -1 or -2, Use PRJs stored on disk
C     Note, unless MODE == -1 or -2, read entire 3D map into memory 

      call system_clock(clock1)
      IF(MODE .GE. 0) THEN
         allocate(MAP3D(NCOLBIN,NROWBIN,NSECBIN), stat = alloc_stat)
         IF(alloc_stat .GT. 0) THEN
            PRINT *, '**********************************************'
            PRINT *, '* Not enough memory for 3D map: EXIT program *'
            CALL TSBEND
         ENDIF
         
c     PIF_READ_WMAPI2 not filling in edges of map3d for BIN=2
c     Initialize to zero to make sure that we don't get NaN's
         if(iam == 0) then
            MAP3D = 0.0         ! F90 array syntax
            CALL PIF_READ_WMAPI2(MAPHEAD, MAP3D, NCOLBIN, NROWBIN,
     $           NSECBIN)
            CALL PIF_CLOSE (1)
         endif
         
         call mpi_bcast(MAP3D, NCOLBIN*NROWBIN*NSECBIN, mpi_real,
     $        0, mpi_comm_world, mpi_err)
         
         basecount = nview_mod/num_nodes
         remainder = nview_mod - basecount*num_nodes
         
         if(iam .ge. (num_nodes-remainder)) then
            sendcount = basecount + 1
         else
            sendcount = basecount
         endif

         recvcounts = 0
         displs = 0
         do iii=0,num_nodes-1
            if(iii .ge. (num_nodes-remainder)) then
               recvcounts(iii+1) = basecount + 1
            else
               recvcounts(iii+1) = basecount
            endif
         enddo
         displs(1) = 0
         do iii=1,num_nodes-1
            displs(iii+1) = displs(iii) + recvcounts(iii)
         enddo

         do id=1,sendcount
            indx = displs(iam+1) + id
            omega = 90.0
            call map_prj(prj, map3d, ncolbin, nrowbin, nsecbin,
     $           mod_ts(indx), mod_ps(indx), omega, 1.0, real(nsecbin))
            prjs(:,:,indx) = prj
         enddo
         
         deallocate(MAP3D)
         
C     Set the size of a projection
         prj_size = ncolbin*nrowbin

C     The following block of code distributes each process's projections
C     to all other processes using a series of nview_mod broadcast
C     operations. This is probably a little slower than the original
C     method of using a single call to mpi_allgatherv, but avoids the
C     problems associated with mpi buffer limits.

         do inode = 0, num_nodes-1
            nprjs = sendcount
            call mpi_bcast(nprjs, 1, mpi_integer, inode, mpi_comm_world,
     $           mpi_err)
            do prj_cnt = 1, nprjs
               call mpi_bcast(prjs(1,1,displs(inode+1)+prj_cnt),
     $              prj_size, mpi_real, inode,
     $              mpi_comm_world, mpi_err)
            enddo
         enddo

C     Original method for distributing projections
CCC   sendcount  = sendcount  * prj_size
CCC   recvcounts = recvcounts * prj_size
CCC   displs     = displs     * prj_size
CCC   call mpi_allgatherv(prjsl, sendcount,
CCC   +        mpi_real, prjs, recvcounts, displs,
CCC   +        mpi_real, mpi_comm_world, mpi_err)


      endif

      call system_clock(clock2)
      tg = elapsed_time(clock1, clock2)

      if(iam == 0) then
         write(*,*) 'Time calculating projections = ',tg, 
     +        ' NCPUs = ', num_nodes
         WRITE(*,1000)
         do iii=1, nview_mod
            WRITE(*,1010) iii, MOD_TS(iii), MOD_PS(iii)
         enddo
      endif

 1000 FORMAT('  View # Theta    Phi',/,21('-'),/)
 1010 FORMAT ('+',I5,F8.2,F7.2)


C  ***********************************
C  *  Determine THETA,PHI,OMEGA,X,Y  *
C  ***********************************

      CALL GLOBAL_CC (MODE, BIN, NROT, NANNULI, NVIEW_MOD,
     1     IDIM1, IDIM2, RES_MIN, RES_MAX, 
     2     PIXRES_MIN, PIXRES_MAX, ANN_LO, ANN_HI, JCUT, RADIUS,
     3     SYM, VERBOSE, MAG_CEN, MAG_STEP, MAG_NUM, 
     4     MOD_TS, MOD_PS, IMG_BKG,
     5     PRJ_AVG, PRJS, PRJ, VAROPT, NCOLBIN, NROWBIN,
     6     NIDS_TOT, IMG_FNAMES, FIDS, IDS, TS, PS, OS, 
     7     XS, YS, MAGS, PFT_CCS, 
     8     PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC, SIGCUT,
     9     XCEN_MODEL, YCEN_MODEL, PFTRAD_LO, PFTRAD_HI, QUICK_OMEGA)

      deallocate(prj_avg, prjs, mod_ts, mod_ps, prj)

      RETURN
      END
