      subroutine cmpt_ctf(fft_klh, hkl_max, fft_ann,
     +     t_analy, mem_cmpt, ctf_val, abs_ctf, disef,
     +     fno, weight)

C     This routine searches 'defocus space' to determine the major
C     defocus, minor defocus,and astigmatism orientation that results in
C     the best overall agreement between the particle images in a
C     micrograph and the model. For the special case of stigmated
C     images, where the major and minor defoci are equal, the search is
C     done over a single defocus parameter and the orientation is kept
C     at zero.
C
C     Although the image/model comparison can be done using any choice
C     of funcmode, the phase method (funcmode=3) works best since it
C     just uses the phase information to determine the optimal location
C     of the CTF nodes.

      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'

C     Subroutine arguments

      integer, intent(in) :: hkl_max, fft_ann, fno
      real, intent(in) :: weight
      complex, intent(in) :: fft_klh(-fft_ann:fft_ann, -fft_ann:fft_ann, 
     $     0:fft_ann)
      real, intent(out) :: ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(out) :: abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in)  :: disef(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      double precision, intent(out) :: mem_cmpt
      double precision, intent(inout) :: t_analy

      integer defmaj_step, defmaj_nangle
      integer defmin_step, defmin_nangle
      integer angle_step,  angle_nangle

      character(len=256) :: linein(max_views)

      integer i, j, k, max_chunk, indx1, indx2
      integer mem_size, start_view, end_view,
     +     num_views, tmp_row, radius, nsize
      integer min_ctfs(3)
      integer alloc_stat, tmp_nangle

      integer, parameter :: ptle_select_flag = 0

      real :: r4_max = huge(r4_max)
      real du, dv
      real center(2),min_value, imgmagfactor, theta, phi, omega,
     +     num_annul, value, min_annul, ctfs(3)
      double precision part_dist
      real    rot_mat(3,3), 
     +     dist(-Nangle:Nangle,-Nangle:Nangle,-Nangle:Nangle),
     +     sum_dist(-Nangle:Nangle,-Nangle:Nangle,-Nangle:Nangle)
      real    orient(0:9, max_views)
      real    pixels_array(npix_row, npix_col)
      logical stigmated
      
      real, allocatable :: rffta_omega(:), rffta(:), rffta_trns(:)
      complex, allocatable :: fc_2d(:,:), fc_2d_intp(:), fc_2d_bak(:)
      
      double precision elapsed_time
      double precision time

      integer*8 clock1, clock2
      
      double precision :: mem_orient = 0.0D0,
     +     mem_rffta_omega = 0.0D0, mem_rffta = 0.0D0 ,
     +     mem_rffta_trns = 0.0D0, mem_fc_2d = 0.0D0 

      character(len=12) :: subname = 'cmpt_ctfr'

C     Set loop bounds for sitgmated and astigmatic cases
      if(delf_majs(fno)==delf_mins(fno)) then
         defmaj_nangle = nangle
         defmin_nangle = 0
         angle_nangle  = 0
         stigmated = .true.
      else
         defmaj_nangle = nangle
         defmin_nangle = nangle
         angle_nangle  = nangle
         stigmated = .false.
      endif

C     Allocate storage for subroutine FFTANALY
      mem_size        = nrffta_row * nrffta_col
      allocate(rffta_omega(nrffta_omega), rffta(mem_size),
     +     rffta_trns(mem_size), stat = alloc_stat)
      
      mem_rffta_omega = dble(4 * nrffta_omega)
      mem_rffta       = dble( 4 * mem_size )
      mem_rffta_trns  = dble( 4 * mem_size )
      mem_orient      = dble(4*10*max_views)
      
      nsize    = (2*Nangle+1)*(2*Nangle+1)*(2*Nangle+1)
      mem_cmpt = mem_orient + mem_rffta_omega 
     +     + mem_rffta + mem_rffta_trns + nsize

      if(alloc_stat > 0) then
         call error_stop (progname, subname, 
     +        ' Memory allocation failed for FFT arrays')
      endif

C     to initialize the value of rffta_omega(nrfft_col+15) that is
C     rffta_omega(nrffta_omega) Initialize the FFT parameters used to
C     transform rows of RFFTA each row has length NRFFTA_COL

      call vrffti( nrffta_col, rffta_omega )

      num_views = 0 
      call system_clock(clock1)

      if(iam == 0) then
         call readorient(orient, max_views, ort_file_name, num_views,
     +        npix_row, npix_col, ptle_select_flag)
         open(unit=18, file = ort_file_name)
         read(18,'(A)') linein(1) ! Read 1st header line and discard
         read(18,'(A)') linein(1) ! Read 2nd header line and discard
         do i=1,num_views
            read(18,'(A)') linein(i) ! Save remaining lines as strings
         enddo
         close(18)
      endif 

      min_value = R4_MAX
      if (Nangle == 0) goto 1111
      
      if(num_nodes > 1) then
         call mpi_bcast(num_views, 1, mpi_integer,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST num_views')
         
         call mpi_bcast(orient, 10*max_views, mpi_real,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST orient')
      endif

      num_annul = real(nrffta_row)*angstrom_per_pix/res_max 
      min_annul = real(nrffta_row)*angstrom_per_pix/res_min
      if (num_annul .ge. nrffta_row/2.) 
     +     call error_stop(progname,subname, 
     +     'Resolution is too high, smaller than 2*angstrom_per_pix')
      if (iam == 0) then
         print *,min_annul,num_annul,Dangle,Nangle
      endif
      radius = int(num_annul)+1

C     In the original version of this routine, the minimum radius used
C     for comparing the images and projections was recomputed on the
C     basis of the defocus value. This can result in the optimal defocus
C     depending on the starting defocus. Although the effect is small,
C     this has been noticed in some cases where the defocus step size is
C     also small. The following block of code has been commented out and
C     retained in case the feature is ever reimplemented.

ccc      real    wavelength, tmaj, fw2
ccc      wavelength = 1.2264264967
ccc     +     /sqrt(VOLTS(fno)+ 0.9784663543E-6 * VOLTS(fno)**2)
ccc      tmaj = min(DELF_MAJS(fno),DELF_MINS(fno))
ccc      fw2 = (tmaj-sqrt(tmaj*tmaj - AMPFACS(fno)*wavelength*2.0))
ccc     +     /AMPFACS(fno)
ccc      min_annul=sqrt(fw2/1000.)*(PIXSIZS(fno)*2*hkl_max)/wavelength/10.
ccc      if (min_annul < 10.) min_annul= 10.
ccc      if (iam == 0) then
ccc         print *, 'change the minimum radius to be', min_annul
ccc      endif

C     read each views and generate 2DDFT for these views
      call set_indices(num_nodes, 1, num_views,
     +         frst_lst_view, max_chunk )

C     allocate memory to store cuts,  DFT bakup,
      mem_size        = ( 1 + nfc_2d_row ) * ( 1 + 2 * nfc_2d_col )
      allocate(fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col), 
     +     fc_2d_intp(mem_size * max_chunk),
     +     fc_2d_bak(mem_size * max_chunk), 
     +     stat = alloc_stat)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname, 
     +        ' Memory allocation failed for fc_2d arrays')
      endif

      mem_fc_2d       = dble( (16 * max_chunk + 8) * mem_size)
      mem_cmpt        = mem_cmpt + mem_fc_2d

      do j = 0, max_chunk - 1
C     read current one view for all processors
         do i = num_nodes - 1 , 0 , -1
            start_view = frst_lst_view(1,i) + j
            end_view   = frst_lst_view(2,i)
            if (start_view <= end_view ) then
C     read one image to each processor one time
               call read_files(pixels_array, start_view,
     +              start_view, i, orient)
            endif
         enddo

         start_view = frst_lst_view(1,iam) + j
         end_view   = frst_lst_view(2,iam)
         if (start_view > end_view) exit !Break out of loop

            theta        = orient(1,start_view)
            phi          = orient(2,start_view)
            omega        = orient(3,start_view)
            center(1)    = orient(4,start_view)
            center(2)    = orient(5,start_view)
            imgmagfactor = orient(6,start_view)

C     adjust the imgmagfactor according to its own pixel_size
         imgmagfactor = imgmagfactor * PIXSIZ_OUT / angstrom_per_pix

C     Compute Discrete Fourier Transform of pixel values:

         call fft_2dfft( pixels_array,
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, 
     +        nrffta_col, nfc_2d_row, 
     +        nfc_2d_col, fc_2d_bak(j*mem_size + 1) )

         du = center(1) - real(int(center(1)))
         dv = center(2) - real(int(center(2)))
         call shift_fft(fc_2d_bak(j*mem_size + 1), 
     +        nfc_2d_row, nfc_2d_col, du, dv)


C     Compute cut for this view:

         call crowther_to_matrix(theta, -phi, -omega, rot_mat)
         call interpl_3d(fft_klh, hkl_max, min_annul, num_annul,
     +        rot_mat, fc_2d_intp(j*mem_size+1), imgmagfactor, fft_ann)
      enddo 

      call system_clock(clock2)
      time = elapsed_time(clock1, clock2)
      t_analy = t_analy + time

C     Initialize entire array of scores to large value
      dist = R4_MAX             ! Fortran 90 array syntax

C     loop to refine CTF, I use min_ctfs and ctfs the temp value of CTF
C     the resolution for defocus is Dangle, and the resolution for angle
C     is Dangle*10

 1110 min_ctfs(1) = 0           ! defocus major
      min_ctfs(2) = 0           ! defocus minor
      min_ctfs(3) = 0           ! the angle
 
      do defmaj_step = -defmaj_nangle, defmaj_nangle  
         ctfs(1)= dangle*defmaj_step + DELF_MAJS(fno)
         if (ctfs(1) <= 0) cycle ! Skip to next iteration
         do defmin_step = -defmin_nangle, defmin_nangle  

            if(stigmated) then
               ctfs(2) = ctfs(1)
            else
               ctfs(2)= dangle*defmin_step + DELF_MINS(fno)
               if (ctfs(2) <= 0) cycle ! Skip to next iteration
            endif

C     If ctfs(1) equals ctfs(2), then don't need to rotate about
C     angle. To avoid having to reproduce a big chunk of code, set the
C     loop indices based on the result of this test.
            if (ctfs(2) == ctfs(1)) then
               tmp_nangle = 0
            else
               tmp_nangle = angle_nangle
            endif

            do angle_step = -tmp_nangle, tmp_nangle
               ctfs(3) = dangle*angle_step*10 + ANG_MAJS(fno)
               if (dist(defmaj_step,defmin_step,angle_step) < 
     +              R4_MAX-1) cycle
               IF (CTFMODE > 0 .and. CTFMODE <= 4)  then
                  call ctf_para(ctf_val, abs_ctf, 
     +                 0, nfc_2d_row, -nfc_2d_col, nfc_2d_col, .true.,
     +                 CTFMODE, Filter, -TEMPFAC, CTF_FF1, CTF_FF2,
     +                 PIXSIZS(fno), units, VOLTS(fno),ctfs(1), 
     +                 ctfs(2), ctfs(3), AMPFACS(fno),
     +                 CS_COEFFS(fno), Weight, res_max,res_max)
               endif

               part_dist = 0.0
               start_view = frst_lst_view(1,iam)
               end_view   = frst_lst_view(2,iam)

               do j = start_view, end_view
                  k = j - start_view
                  tmp_row = 0
                  do indx2 = -nfc_2d_col, nfc_2d_col
                     do indx1 = 0, nfc_2d_row
                        tmp_row = tmp_row + 1
                        fc_2d(indx1, indx2) = 
     +                       fc_2d_bak(k*mem_size + tmp_row)
     +                       * ctf_val(indx1, indx2)
                     enddo
                  enddo

                  call imgcomport(fc_2d_intp(k*mem_size+1), 
     +                 hkl_max, fc_2d, disef, abs_ctf, 
     +                 min_annul, num_annul, value, funcmode)
                  part_dist = part_dist + value
               enddo

               dist(defmaj_step, defmin_step, angle_step) = part_dist

            enddo
         enddo
      enddo
      
      call MPI_Allreduce(dist, sum_dist, nsize,
     +     MPI_REAL, MPI_SUM, MPI_COMM_WORLD, mpi_err)

C     Determine location of minimum element in array sum_dist. Note use
C     of Fortran 90 array syntax and intrinsic function minloc (which
C     behaves as if lower bounds of array are all equal to 1)

      min_ctfs = minloc(sum_dist)
      min_ctfs = min_ctfs - nangle - 1

      if ( abs(real(min_ctfs(1))) > Nangle*0.5 .or. 
     +     abs(real(min_ctfs(2))) > Nangle*0.5 .or.
     +     abs(real(min_ctfs(3))) > Nangle*0.5 ) then
         if(stigmated) then
            DELF_MAJS(fno) = DELF_MAJS(fno) + min_ctfs(1) * Dangle
            DELF_MINS(fno) = DELF_MAJS(fno)
            ANG_MAJS(fno)  = 0.0
         else
            DELF_MAJS(fno) = DELF_MAJS(fno) + min_ctfs(1) * Dangle
            DELF_MINS(fno) = DELF_MINS(fno) + min_ctfs(2) * Dangle
            ANG_MAJS(fno)  = ANG_MAJS(fno)  + min_ctfs(3) * Dangle * 10
         endif
         
C     slide dist windows to make min dist value is in center
         call recenter_array_3d(dist, Nangle, min_ctfs, R4_MAX)
         
         goto  1110
      endif

      if(stigmated) then
         DELF_MAJS(fno) = DELF_MAJS(fno) + min_ctfs(1) * Dangle
         DELF_MINS(fno) = DELF_MAJS(fno)
         ANG_MAJS(fno)  = 0.0
      else
         DELF_MAJS(fno) = DELF_MAJS(fno) + min_ctfs(1) * Dangle
         DELF_MINS(fno) = DELF_MINS(fno) + min_ctfs(2) * Dangle
         ANG_MAJS(fno)  = ANG_MAJS(fno)  + min_ctfs(3) * Dangle * 10
      endif

C     write out new CTF and orient data
 1111 if(iam == 0) then

         write(*,1001) iam, ort_file_name, num_views, max_views
 1001    format(/' Node = ',i2,':  Read orientation file: ',a/,
     +        12x,' Number of views     = ', i9, i9)
         
         open (unit=18, file=ort_file_name)
         write(18,'(A)') trim(adjustl(pix_file_name))
         write(18,1010) pixsizs(fno), units, volts(fno),
     $        ampfacs(fno), delf_majs(fno), delf_mins(fno),
     $        ang_majs(fno), cs_coeffs(fno)
         do i=1,num_views
            write(18,*)trim(linein(i)) ! Write out rest of original file
         enddo
         close(18)

 1010    FORMAT (1X,F8.4,',',I2,',',F10.1,',',4(F10.4,','),F8.2) 
      endif
      
      deallocate(rffta_omega, rffta, rffta_trns,
     +     fc_2d, fc_2d_intp, fc_2d_bak)
      
      return
      end subroutine cmpt_ctf
